﻿// USB HID Scale Reader by Matt Galanto using HidLibrary.dll by Mike O'Brien
//  http://www.bumderland.com/dev

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using HidLibrary;

namespace PostageScale
{
    public partial class Form1 : Form
    {
        //
        // Member data
        //
        HidDevice[] mahdDevices;
		HidDevice mhdDevice;

        public Form1()
        {
            InitializeComponent();
        }

		//
		// Private methods
		//
		private bool FailNoDevice()
		{
			if(mhdDevice == null)
			{
				MessageBox.Show("No Device Selected", "Error", MessageBoxButtons.OK,
					MessageBoxIcon.Error);
				return true;
			}

			return false;
		}

		private bool FailDeviceClosed()
		{
			if(!mhdDevice.IsOpen)
			{
				MessageBox.Show("Device is not open.", "Error", MessageBoxButtons.OK,
					MessageBoxIcon.Error);
				return true;
			}

			return false;
		}

		private void AddTextToReadBox(string vsText)
		{
			if(chkKeep.Checked && txtRead.TextLength > 0)
				txtRead.Text = txtRead.Text + "\r\n" + vsText;
			else
				txtRead.Text = vsText;

			txtRead.Select(txtRead.TextLength, 0);
			txtRead.ScrollToCaret();
		}

		// returns 16 on error
		private byte GetHexValue(char vcChar)
		{
			if(vcChar >= '0' && vcChar <= '9')
				return (byte)(vcChar - '0');
			if(vcChar >= 'A' && vcChar <= 'F')
				return (byte)(vcChar - 'A' + 10);
			if(vcChar >= 'a' && vcChar <= 'f')
				return (byte)(vcChar - 'a' + 10);
			return 16;
		}

		//
		// Events
		//
        private void Form1_Load(object sender, EventArgs e)
        {
			mahdDevices = HidDevices.Enumerate().ToArray();
			mhdDevice = null;

            for(int i = 0; i < mahdDevices.Length; i++)
            {
				cmbDevices.Items.Add(mahdDevices[i].Description);
            }
        }

		private void cmbDevices_SelectedIndexChanged(object sender, EventArgs e)
		{
			mhdDevice = mahdDevices[cmbDevices.SelectedIndex];

			if(mhdDevice.IsOpen)
				btnOpen.Text = "Close";
			else
				btnOpen.Text = "Open";
		}

		private void btnOpen_Click(object sender, EventArgs e)
		{
			if(FailNoDevice())
				return;

			if(mhdDevice.IsOpen)
			{
				mhdDevice.CloseDevice();
				btnOpen.Text = "Open";
			}
			else
			{
				mhdDevice.OpenDevice();
				btnOpen.Text = "Close";
			}
		}

		private void btnRead_Click(object sender, EventArgs e)
		{
			if(FailNoDevice() || FailDeviceClosed())
				return;

			HidDeviceData hddData = mhdDevice.Read();
			AddTextToReadBox(BitConverter.ToString(hddData.Data));
		}

		private void btnReadScale_Click(object sender, EventArgs e)
		{
			if(FailNoDevice() || FailDeviceClosed())
				return;

			HidDeviceData hddData = mhdDevice.Read();
			if(hddData.Data.Length != 6)
			{
				MessageBox.Show("Invalid scale data", "Error", MessageBoxButtons.OK,
					MessageBoxIcon.Error);
				AddTextToReadBox(BitConverter.ToString(hddData.Data));
			}
			else
			{
				AddTextToReadBox("Weight: " + ((hddData.Data[4] | (hddData.Data[5]
					<< 8)) / 10.0) + " oz");
			}
		}

		private void btnSend_Click(object sender, EventArgs e)
		{
			if(FailNoDevice() || FailDeviceClosed())
				return;

			bool bFirstDigitRead = false;
			byte nDigit = 0;
			byte[] anBytes = new byte[txtSend.TextLength / 2];
			int nByteIndex = 0;
			for(int i = 0; i < txtSend.TextLength; i++)
			{
				nDigit = GetHexValue(txtSend.Text[i]);
				if(nDigit > 15)
				{
					// Non hex digit
					if(bFirstDigitRead)
					{
						nByteIndex++;
						bFirstDigitRead = false;
					}
				}
				else
				{
					if(bFirstDigitRead)
					{
						anBytes[nByteIndex] <<= 4;
						anBytes[nByteIndex] |= nDigit;
						nByteIndex++;
						bFirstDigitRead = false;
					}
					else
					{
						anBytes[nByteIndex] = nDigit;
						bFirstDigitRead = true;
					}
				}
			}

			// See if last byte is a single char
			if(bFirstDigitRead)
				nByteIndex++;

			byte[] anData = new byte[nByteIndex];

			//
			// TODO: CopyTo requires arrays of same length, find another way
			//

			//anBytes.CopyTo(anData, 0);
			Array.Copy(anBytes, anData, nByteIndex);
			mhdDevice.Write(anData);
			//AddTextToReadBox(BitConverter.ToString(anData));
		}

		
    }
}
